/*
 * This file is part of "VirtuaMUnstaz Demo Distribution".
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * "VirtuaMUnstaz Demo Distribution" is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: vms.h,v 2.2 2000/03/02 06:52:10 leo Exp $ */

/*
 * Most of the information found here has been collected by marcus@idonex.se
 * and may be obtained here: http://marcus.mangakai.com/dc
 */

#ifndef _VMS_H_
#define _VMS_H_

/* vms offsets (excluding file header start) */
#define VMS_VMS_DESCR   0x00
#define VMS_ROM_DESCR   0x10
#define VMS_APPLICATION 0x30
#define VMS_ICON_COUNT  0x40
#define VMS_ANIM_SPEED  0x42
#define VMS_EYECATCH    0x44
#define VMS_CHECKSUM    0x46 /* ignored for game data */
#define VMS_DATA_SIZE   0x48 /* ignored for game data */
#define VMS_PALETTE     0x60
#define VMS_ICONS       0x80

struct vms {
  char vmu_description[16];	/* description for VMU file menu */
  char rom_description[32];	/* description for DC boot ROM file manager */
  char creator[16];		/* application identifier */
  char icon_count[2];		/* number of icons (>1 for animated icons) */
  char anim_speed[2];		/* icon animation speed */
  char eyecatch[2];		/* graphic eyecatch (0 for none) */
  char checksum[2];		/* data CRC (not for game files) */
  char data_size[4];		/* actual data size after all header data */
  char reserved[20];

  char palette[32];		/* icon palette (16 16bit integer) */
  char *icons;			/* pointer to icon bitmaps */
  char *ec_palette;		/* pointer to eyecatch palette */
  char *ec_bitmap;		/* pointer to eyecatch bitmap */
};

#define EYECATCH_NONE	0
#define EYECATCH_TRUE	1
#define EYECATCH_256	2
#define EYECATCH_16	3

#endif _VMS_H_
